/*
 * Name: raprpc.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-10-10
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: raprpc.h,v 1.1.1.1 1999/04/12 13:56:34 cs Exp $
 */

#ifndef __raprpc_H_INCLUDED__
#define __raprpc_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
The Remote Administration Protocol calls allow the client to retrieve useful
information from the server, e.g. a list of available shares. The RAP is
layered on top of the SMB_COM_TRANSACT smb, which is implemented in smb.c.
This module implements the RAP calls needed.
*/

/* ------------------------------------------------------------------------- */

struct smbRequest;

typedef struct rapEnumEntry{
	btint	type;
	btu8	*name;
	btu8	*remark;
}rapEnumEntry_t;

typedef struct rapRes{
	btError			err;
	btu16			status;
	btint			numEntries;
	rapEnumEntry_t	*entries;
}rapRes_t;

/* ------------------------------------------------------------------------- */

#define	RAP_SHARE_TYPE_DISKTREE	0
#define	RAP_SHARE_TYPE_PRINTQ	1
#define	RAP_SHARE_TYPE_DEVICE	2
#define	RAP_SHARE_TYPE_IPC		3

/* stati: */
#define	RAP_OK	0
#define	RAP_ERR_ACCESS_DENIED			5
#define	RAP_ERR_NETWORK_ACCESS_DENIED	65
#define	RAP_ERR_MORE_DATA				234
#define	RAP_ERR_ServerNotStarted		2114
#define	RAP_ERR_BadTransactConfig		2141

/* ------------------------------------------------------------------------- */

void	rapEnumShares(struct smbRequest *smb, btu16 uid, btu16 tid, void *caller);
void	rapEnumServers(struct smbRequest *smb, btu8 *workgroup, btbool listDomains, btu16 uid, btu16 tid, void *caller);

/* ------------------------------------------------------------------------- */

#endif /* __raprpc_H_INCLUDED__ */
