/*
 * Name: user.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-28
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: user.h,v 1.3 1999/07/22 17:54:12 cs Exp $
 */

#ifndef __user_H_INCLUDED__
#define __user_H_INCLUDED__

#include "basictypes.h"
#include "array.h"	/* for collectionObjectI_t */

/*
General Description:
This module implements the class describing a user (local _and_ remote).
It also implements the procedure of user login at the remote server.
User objects are stored at the server where the user is logged in.
*/

/* ------------------------------------------------------------------------- */

struct server;
struct share;

typedef struct cifsUser{
    btu32			unixUid;
    btu32			unixGid;
    struct server	*server;
    btuni			*smbUsername;
    bti32			refCount;
	btuni			*plainPasswd;
	btu8			pwHashPreNt[16];
	btu8			pwHashNt[16];
    btu16			smbUid;				/* UID negotiated for this user */
	btu8			allowGuest;			/* whether guest connects are OK */
    btbool			isGuest;
	btbool			isInvalidated;		/* should be freed when possible */
    int				cstatus;			/* connection status */
    struct procCond	*connectSignal;		/* only one connect at a time */
}cifsUser_t;

typedef struct userRes{
	btError		err;
	cifsUser_t	*user;
	btbool		userIsNew;
}userRes_t;

extern collectionObjectI_t	userByUnixUidInterface;

cifsUser_t	*userNew(btu32 unixUid, btuni *smbUsername, btuni *plainPasswd, btu8 passHashNt[16], btu8 passHashPreNt[16], btbool allowUnencrypted);
void	userFree(cifsUser_t *this);
void	userConnectionLost(cifsUser_t *this);
void	userConnect(cifsUser_t *this, void *caller);
void	userDisconnect(cifsUser_t *this, void *caller);
void	userAuthenticate(struct server *server, btint unixUid, void *caller);
/* returns userRes_t with res->user containing a retain-neutral user object. Caller should not release unless the user object should be freed. */
btbool	userUnixUidsAreEqual(cifsUser_t *this, cifsUser_t *other);
btu32	userUnixUidHashval(cifsUser_t *this);
void	userRetain(cifsUser_t *this);
void	userRelease(cifsUser_t *this);
void	userInvalidate(cifsUser_t *user);

/* ------------------------------------------------------------------------- */

static inline btu16	userSmbUid(cifsUser_t *this)
{
    return this->smbUid;
}

static inline btu32	userUnixUid(cifsUser_t *this)
{
    return this->unixUid;
}

static inline btu32	userUnixGid(cifsUser_t *this)
{
	return this->unixGid;
}

static inline int	userConnectionStatus(cifsUser_t *this)
{
    return this->cstatus;
}

static inline btuni	*userCifsName(cifsUser_t *this)
{
	return this->smbUsername;
}

/* ------------------------------------------------------------------------- */

#endif	/* __user_H_INCLUDED__ */
