/*
 * Name: argparser.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-12-08
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: argparser.h,v 1.1.1.1 1999/04/12 13:56:37 cs Exp $
 */

#ifndef __argparser_H_INCLUDED__
#define __argparser_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements a general purpose argument parser similar to the
standard getopt() function. The intended use is to first allocate an
argParser object, define the allowed options with a sequence of
argpAddOption() calls (also assigning help text if available) and then
parse the commandline with argpParse(). This function returns 1 on success
and 0 on failure, printing errors to 'errFp' (a FILE *). After successful
parsing, the arguments are available through the argpOption*() functions.
*/

/* ------------------------------------------------------------------------- */

struct hashtable;
struct array;

typedef struct argParser{
	struct hashtable	*optByName;	/* for all kinds of querying */
	struct array		*optList;	/* sorted by order of adding, for help */
	struct array		*arguments;	/* unnamed arguments */
}argParser_t;

/* ------------------------------------------------------------------------- */

argParser_t	*argpNew(void);
void	argpFree(argParser_t *ap);
void	argpAddOption(argParser_t *ap, btu8 name, btu8 *paramName, btbool isNumeric, btu8 *helpText);
btbool	argpParse(argParser_t *ap, btint argc, char **argv, void *errFp);
void	argpPrintHelp(argParser_t *ap, void *outFd);
btint	argpArgumentCount(argParser_t *ap);
btu8	*argpArgumentAtIndex(argParser_t *ap, btint index);
btbool	argpOptionIsSet(argParser_t *ap, btu8 optName);
btint	argpOptionIntVal(argParser_t *ap, btu8 optName, btbool *isValid);
btu8	*argpOptionStringVal(argParser_t *ap, btu8 optName);

/* ------------------------------------------------------------------------- */

#endif /* __argparser_H_INCLUDED__ */
