/*
 * Name: errcodes.c
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-10
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: errcodes.c,v 1.4 1999/07/22 17:55:25 cs Exp $
 */

/* Note: This file is also parsed during creation of the manual. Each
 * return-line may be followed by a remark explaining the error in more
 * detail. The returned text and the remark are extracted into the manual.
 */

#include "stdheaders.h"
#include "system.h"
#include "errcodes.h"


btu8	*errDescription(btError *err)
{
static btu8	scratch[256];

    switch(err->class){
    case ERRCLS_OK:						return "No Error";
    case ERRCLS_CIFS_DOS:
        switch(err->code){
		case ERR_CIFS_DOSbadfunc:		return "DOS: Bad Function";
		case ERR_CIFS_DOSbadfile:		return "DOS: Bad File";
		case ERR_CIFS_DOSbadpath:		return "DOS: Bad Path";
		case ERR_CIFS_DOSnofids:		return "DOS: File table overflow";
		case ERR_CIFS_DOSnoaccess:		return "DOS: Access denied";
/* If you get this error during login, the server rejected your password. If you try to login
 * to a domain account, Windows NT sometimes gives this error although the password is
 * correct. If that happens, the password may be expired or not valid now. If you recently
 * changed it, the old password may still work. */
		case ERR_CIFS_DOSbadfid:		return "DOS: Bad file descriptor";
		case ERR_CIFS_DOSbadmcb:		return "DOS: Bad MCB";
		case ERR_CIFS_DOSnomem:			return "DOS: No memory";
		case ERR_CIFS_DOSbadmem:		return "DOS: Bad memory";
		case ERR_CIFS_DOSbadenv:		return "DOS: Bad environment";
		case ERR_CIFS_DOSbadformat:		return "DOS: Bad format";
		case ERR_CIFS_DOSbadaccess:		return "DOS: Bad access";
		case ERR_CIFS_DOSbaddata:		return "DOS: Bad data";
		case ERR_CIFS_DOSbaddrive:		return "DOS: Bad drive";
		case ERR_CIFS_DOSremcd:			return "DOS: remcd";
		case ERR_CIFS_DOSdiffdevice:	return "DOS: Different device";
		case ERR_CIFS_DOSnofiles:		return "DOS: No more files";
		case ERR_CIFS_DOSbadshare:		return "DOS: Bad share";
		case ERR_CIFS_DOSlock:			return "DOS: Operation would block";
		case ERR_CIFS_DOSnotshared:		return "DOS: Share not exported by server";
		case ERR_CIFS_DOSfilexists:		return "DOS: File exists";
		case ERR_CIFS_DOSnotempty:		return "DOS: Directory not empty";
            /* the following are heuristic values derived from smbfs */
		case 87:						return "DOS: error 87";
		case ERR_CIFS_DOSdestexists:	return "DOS: error 183: destination exists";
		default:
			sprintf(scratch, "Unknown DOS error %d/%d", err->class, err->code);
			return scratch;
        }
        break;
    case ERRCLS_CIFS_SRV:
        switch(err->code){
		case ERR_CIFS_SRVerror:			return "SRV: General error";
		case ERR_CIFS_SRVbadpw:			return "SRV: Bad password";
/* If you get this error during login, the server rejected your password. See error
 * <em>DOS: Access denied</em> for more information. */
		case ERR_CIFS_SRVaccess:		return "SRV: Access denied";
/* If you get this error during login, the server rejected your password. See error
 * <em>DOS: Access denied</em> for more information. */
		case ERR_CIFS_SRVinvnid:		return "SRV: Invalid NID";
		case ERR_CIFS_SRVinvnetname:	return "SRV: Invalid netname";
		case ERR_CIFS_SRVinvdevice:		return "SRV: Invalid device";
		case ERR_CIFS_SRVqfull:			return "SRV: Queue full";
		case ERR_CIFS_SRVqtoobig:		return "SRV: Queue too big";
		case ERR_CIFS_SRVqeof:			return "SRV: Queue at EOF";
		case ERR_CIFS_SRVinvpfid:		return "SRV: Invalid PFID";
		case ERR_CIFS_SRVsmbcmd:		return "SRV: SMB cmd";
		case ERR_CIFS_SRVsrverror:		return "SRV: Server error";
		case ERR_CIFS_SRVfilespecs:		return "SRV: File specs";
		case ERR_CIFS_SRVbadpermits:	return "SRV: Bad permits";
		case ERR_CIFS_SRVsetattrmode:	return "SRV: Set attributes mode";
		case ERR_CIFS_SRVpaused:		return "SRV: Paused";
		case ERR_CIFS_SRVmsgoff:		return "SRV: Message off";
		case ERR_CIFS_SRVnoroom:		return "SRV: No room";
		case ERR_CIFS_SRVrmuns:			return "SRV: rmuns";
		case ERR_CIFS_SRVtimeout:		return "SRV: Timeout";
		case ERR_CIFS_SRVnoresource:	return "SRV: No resource";
		case ERR_CIFS_SRVtoomanyuids:	return "SRV: Too many users";
		case ERR_CIFS_SRVbaduid:		return "SRV: Bad user-ID";
		case ERR_CIFS_SRVusempx:		return "SRV: Use multiplexed";
		case ERR_CIFS_SRVusestd:		return "SRV: Use standard";
		case ERR_CIFS_SRVcontmpx:		return "SRV: Continue multiplexed";
		case ERR_CIFS_SRVnosupport:		return "SRV: Not supported";
		default:
			sprintf(scratch, "Unknown SRV error %d/%d", err->class, err->code);
			return scratch;
        }
        break;
    case ERRCLS_CIFS_HARD:
        switch(err->code){
		case ERR_CIFS_HRDnowrite:		return "HRD: No write";
		case ERR_CIFS_HRDbadunit:		return "HRD: Bad unit";
		case ERR_CIFS_HRDnotready:		return "HRD: Not ready";
		case ERR_CIFS_HRDbadcmd:		return "HRD: Bad command";
		case ERR_CIFS_HRDdata:			return "HRD: Data";
		case ERR_CIFS_HRDbadreq:		return "HRD: Bad request";
		case ERR_CIFS_HRDseek:			return "HRD: Seek";
		case ERR_CIFS_HRDbadmedia:		return "HRD: Bad media";
		case ERR_CIFS_HRDbadsector:		return "HRD: Bad sector";
		case ERR_CIFS_HRDnopaper:		return "HRD: No paper in printer";
		case ERR_CIFS_HRDwrite:			return "HRD: Write";
		case ERR_CIFS_HRDread:			return "HRD: Read";
		case ERR_CIFS_HRDgeneral:		return "HRD: General";
		case ERR_CIFS_HRDbadshare:		return "HRD: Bad share";
		case ERR_CIFS_HRDlock:			return "HRD: Operation would block";
		case ERR_CIFS_HRDwrongdisk:		return "HRD: Wrong disk";
		case ERR_CIFS_HRDFCBUnavail:	return "HRD: FCB unavailable";
		case ERR_CIFS_HRDsharebufexc:	return "HRD: Share buffer exceeded";
		default:
			sprintf(scratch, "Unknown HRD error %d/%d", err->class, err->code);
			return scratch;
        }
        break;
    case ERRCLS_FRAMEWORK:
        switch(err->code){
		case ERR_FRWmodulenotfound:		return "FRW: Module not found";
/* A module specified by name could not be found. If you had to type a module name,
 * check the spelling. */
		case ERR_FRWmoduletypeerr:		return "FRW: Module type mismatch";
/* The module you specified has the wrong type. You must specify a backend module for the
 * backend and a frontend module for the frontend. */
		case ERR_FRWalreadymounted:		return "FRW: Mountpoint already in use";
/* The path you gave for the mountpoint is already mounted by Sharity. You can't over-mount
 * a Sharity mount with an other Sharity mount. */
		case ERR_FRWipcparameters:		return "FRW: Internal error: IPC parameters";
/* One of the user interface programs created illegal parameters. */
		case ERR_FRWipcversion:			return "FRW: Internal error: IPC protocol version";
/* There's a protocol mismatch between the user interface program and the daemon. You MUST
 * use a consistent installation with user interface and daemon from the same version. */
		case ERR_FRWnomem:				return "FRW: Internal error: Out of Memory";
/* Sharity has not been able to allocate enough memory. You should probably increase the size
 * of your swap area. Sharity may get into troubles when it can't allocate enough memory if it
 * needs some. You should restart the daemon if you get this error. */
		case ERR_FRWnosuchobj:			return "FRW: Internal error: Referenced GUI object not found";
/* The user interface tried to reference an object (such as a server, a login or whatever) which
 * is not known to the daemon. You should probably restart the user interface because there's
 * obviously a status mismatch. */
		case ERR_FRWnosuchmethod:		return "FRW: Internal error: Referenced GUI method not found";
/* The user interface tried to execute a function which is not available in the daemon. This must
 * be a programming error. Please send a bugreport to Objective Development. */
		case ERR_FRWumountperm:			return "FRW: You don\'t have permissions to unmount";
/* You tried to unmount a mount which was mounted by somebody else. Only the owner of a mount and
 * root can unmount it. */
		case ERR_FRWclientcomm:			return "FRW: Internal error: Communication with UI client";
/* It's very unlilely that you ever see this message. It indicates that the user interface (which
 * displays error messages to you) terminated or crashed. */
		case ERR_FRWnosuchmnt:			return "FRW: Path not mounted";
/* The mountpoint which you tried to unmount is not mounted by Sharity. If you think it should be,
 * please check your spelling or look at the mountpoint listing created by <tt>cifslist</tt> */
		case ERR_FRWmpaccessperm:		return "FRW: Not owner of mount, operation denied";
/* You tried to store a mount or delete a stored mount that belongs to somebody else. You may only
 * manipulate mountpoints which you have mounted yourself. Only root may manipulate all mounts. */
		case ERR_FRWautomounted:		return "FRW: Operation denied because mount was done by automounter";
/* You tried to store an automounted mountpoint. This operation is forbidden because it's not useful. */
		case ERR_FRWipcnotimpl:			return "FRW: Internal error: IPC function not implemented";
/* This error message indicates a programming error. Please send a bugreport to Objective
 * Development if it occurs. */
		case ERR_FRWusrabort:			return "FRW: Abort by user";
/* An operation (e.g. a login) which needed more information from the user could not succeed because
 * the user did not provide that information. */
 		case ERR_FRWnotauth:			return "FRW: You are not authorized to perform this operation";
/* You tried to add or remove a server or browser from a list, but you are not root and changing
 * the configuration has not been allowed for non-root users. Run the GUI application as root
 * and change the configuration as desired or allow non-root users to change the configuration. */
 		case ERR_FRWnomountpoint:		return "FRW: Mount point does not exist";
/* You attempted a mount, but the mount point directory does not exist. Create the
 * directory before you mount. */
        default:
			sprintf(scratch, "Unknown FRW error %d/%d", err->class, err->code);
			return scratch;
        }
        break;
    case ERRCLS_CIFS_LOCAL:
        switch(err->code){
		case ERR_CIFS_LOCconnlost:		return "LOC: Connection lost";
/* The connection to the CIFS server was lost, probably due to a network problem. */
		case ERR_CIFS_LOCtimeout:		return "LOC: Timeout";
/* An operation did not proceed within the time limit, probably due to a network problem. */
		case ERR_CIFS_LOCbogusdata:		return "LOC: Bogus data received";
/* The CIFS server sent data which could not be interpreted according to the specification. */
		case ERR_CIFS_LOCstale:			return "LOC: Stale NFS handle";
/* A file has been accessed which does not exist any more. The file may have been deleted from the
 * server. */
		case ERR_CIFS_LOCpwencrypt:		return "LOC: Server can\'t encrypt passwords, use option to override";
/* The server can't use CIFS password encryption. Sharity does not allow unencrypted passwords by
 * default (for security reasons). If you need to interoperate with this server, you can allow
 * sending the password unencrypted with an option. */
		case ERR_CIFS_LOClookup:		return "LOC: Netbios name lookup failure";
/* The netbios name has not been found. Either the named host does not exist or you have not
 * configured the WINS setting correctly. */
		case ERR_CIFS_LOClicclients:	return "LOC: License exceeded, too many clients";
/* You are trying to use Sharity on more clients than the license allows. You should upgrade to a
 * bigger license or shut down Sharity on other clients. If you want to know where the other licenses
 * are running, type <tt>cifslicense -ll</tt> in a terminal window to see the license utilization.
 * It may take several minutes until a change in the license utilization propagates to all clients. */
		case ERR_CIFS_LOClicservers:	return "LOC: License exceeded, too many servers";
/* You are trying to mount more servers than the license allows. Servers are counted network wide,
 * not per client. To see which servers are mounted by which client, type <tt>cifslicense -ll</tt>
 * in a terminal window to see the license utilization.
 * It may take several minutes until a change in the license utilization propagates to all clients. */
		case ERR_CIFS_LOClicense:		return "LOC: License string is invalid";
/* The license you entered is either not a valid license string or it's an expired time-limited license. */
		case ERR_CIFS_LOCsrvconn:		return "LOC: Can\'t set license when servers are connected";
/* Due to the inner workings of the licensing scheme, the license can only be set when no server is
 * connected. Unmount all shares and log out all users before you set the license. If this does not
 * work, delete all stored mounts and logins and restart Sharity. */
		case ERR_CIFS_LOCwrongphrase:	return "LOC: Pass Phrase not accepted";
/* The pass phrase you entered was not correct. */
		case ERR_CIFS_LOCneedpassphrase: return "LOC: Server requires Pass Phrase";
/* You need a pass phrase in order to connect to this SSL CIFS server, because your private SSL
 * key is encrypted. */
		case ERR_CIFS_LOCnetbiosrefused: return "LOC: Netbios session refused";
/* The server rejected the Netbios connection. A possible reason is that the server name you used
 * does not match the server's netbios name. The server's event log should provide more information. */
		case ERR_CIFS_LOChavenossl:		return "LOC: This Sharity has been compiled without SSL support";
/* You are using a Sharity version without SSL support, but the server (or an option) requires SSL. */
		case ERR_CIFS_LOCnosslserver:	return "LOC: Server can\'t use SSL encryption";
/* You set an option to require encrypted SSL data transport, but the server can't do SSL. */
		case ERR_CIFS_LOCsslerror:		return "LOC: An SSL error occured, see the syslog for details";
/* The SSLeay library, which is responsible for the encryption, reported an error. Details can be found
 * in your syslog. The path of the syslog file depends on your system's configuration. See the
 * configuration file <tt>/etc/syslog.conf</tt> and the related manual page if you can't find your
 * syslog. */
		case ERR_CIFS_LOCnomountdir:	return "LOC: The browser\'s mount path does not exist";
/* Browsers need a private directory where they can automatically mount shares. This directory is not
 * automatically created on demand (although the share mountpoints inside it are). If it does not exist,
 * this error is given. */
		case ERR_CIFS_LOCnobrowser:		return "LOC: No browser configuration with the given name found";
/* You have tried to mount a browser which is not configured. Compare your spelling with the list of
 * browsers in the GUI or in the configuration file. */
		case ERR_CIFS_LOCotherlogin:	return "LOC: You are not owner of the login";
/* You have tried to log out an other user or to store or delete an other user's login. Only root
 * is allowed to manipulate other users' logins. */
		case ERR_CIFS_LOCsslserver:		return "LOC: Server requests SSL";
/* The server requires a secure connection (SSL), but your Sharity is compiled without SSL support. */
		case ERR_CIFS_LOCsrvnotconn:	return "LOC: Server not connected";
/* You tried to log out from a server which has no active connections within Sharity. Check the
 * spelling of the server and view the current logins with <tt>cifslist</tt>. */
		case ERR_CIFS_LOCusrnotconn:	return "LOC: User not logged in";
/* You tried to log out from a server where you are not logged in. */
        default:
			sprintf(scratch, "Unknown LOC error %d/%d", err->class, err->code);
			return scratch;
        }
        break;
    case ERRCLS_UNIX:
		sprintf(scratch, "UNIX: %s", sysStrerror(err->code));
		return scratch;
        break;
    default:
		sprintf(scratch, "Unknown error %d/%d", err->class, err->code);
		return scratch;
    }
}

