/*
 * Name: fifo.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-11-17
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: fifo.h,v 1.1.1.1 1999/04/12 13:56:37 cs Exp $
 */

#ifndef __fifo_H_INCLUDED__
#define __fifo_H_INCLUDED__

#include "basictypes.h"
#include "memalloc.h"

/*
General Description:
This module (consisting only of this header file, BTW) implements a "first-in
first-out" container data type. The objects that may be added to a fifo must
be a subclass of 'fifoObject_t'. This means that they must have a variable
of type 'fifoObject_t' as the first instance variable (at the offset 0 from
the object pointer).

The function prototypes of the methods implemented by this class should be
sufficient documentation.
*/

/* ------------------------------------------------------------------------- */

typedef struct fifoObject{
    struct fifoObject	*next;
}fifoObject_t;

typedef struct fifo{
    fifoObject_t	*head;
    fifoObject_t	**tail;
}fifo_t;

/* ------------------------------------------------------------------------- */

static inline fifo_t	*fifoNew(void)
{
fifo_t	*this = memCalloc(sizeof(*this));

	this->tail = &this->head;
	return this;
}

static inline void	fifoFree(fifo_t *this)
{
	memFree(this);
}

static inline void	fifoAddObject(fifo_t *this, void *obj)
{
fifoObject_t	*p = obj;

    *this->tail = p;
    p->next = NULL;
    this->tail = &p->next;
}

static inline void	*fifoGetObject(fifo_t *this)
{
fifoObject_t	*p;

    p = this->head;
	if(this->head != NULL){
		this->head = p->next;
		if(this->tail == &p->next){	/* removing the last entry */
			this->tail = &this->head;
		}
		p->next = NULL;
    }
	return p;
}

static inline btbool	fifoIsEmpty(fifo_t *this)
{
    return this->head == NULL;
}

static inline void	fifoClear(fifo_t *this)
{
    this->head = NULL;
    this->tail = &this->head;
}

/* ------------------------------------------------------------------------- */

#endif	/* __fifo_H_INCLUDED__ */
