/*
 * Name: memalloc.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-08-01
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: memalloc_debug.c,v 1.1 1999/08/01 17:41:21 cs Exp $
 */

#include "stdheaders.h"
#include "memalloc.h"
#include "logging.h"

/* ------------------------------------------------------------------------- */

EXC_DEFINE(memExcNomem);

#define	CONST_MALLOCED	0xf2887735
#define	CONST_FREED		0x99bf7ac9

/* ------------------------------------------------------------------------- */

void	*memAlloc(btsize size)
{
btu32	*p;

	if(size <= 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_RESOURCE))
			logPrintf(LOGM_RESOURCE, "allocating %d bytes\n", size);
	}
	size += 4;
	if((p = malloc(size)) == NULL){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "error allocating %d bytes of memory\n", size);
		EXC_THROW(memExcNomem);
	}
	p[0] = CONST_MALLOCED;
	return p + 1;
}

/* ------------------------------------------------------------------------- */

void	*memCalloc(btsize size)
{
btu32	*p;

	if(size <= 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_RESOURCE))
			logPrintf(LOGM_RESOURCE, "callocating %d bytes\n", size);
	}
	size += 4;
	if((p = calloc(size, 1)) == NULL){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "error allocating %d bytes of memory\n", size);
		EXC_THROW(memExcNomem);
	}
	p[0] = CONST_MALLOCED;
	return p + 1;
}

/* ------------------------------------------------------------------------- */

void	*memRealloc(void *oldbase, btsize newsize)
{
btu32	*p, *oldp = oldbase;

	if(newsize <= 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_RESOURCE))
			logPrintf(LOGM_RESOURCE, "reallocating to %d bytes\n", newsize);
	}
	newsize += 4;
    if(oldp == NULL){
        p = malloc(newsize);
    }else{
		if(oldp[-1] != CONST_MALLOCED){
			logPrintf(LOGM_RESOURCE, "realloc: block not allocated\n");
			DEBUG_TRAP();
		}
		oldp[-1] = CONST_FREED;
        p = realloc(oldp - 1, newsize);
    }
	if(p == NULL){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "error reallocating to %d bytes\n", newsize);
		EXC_THROW(memExcNomem);
	}
	p[0] = CONST_MALLOCED;
	return p + 1;
}

/* ------------------------------------------------------------------------- */

void	memFree(void *base)
{
btu32	*p = base;

	if(base == NULL){
		logPrintf(LOGM_RESOURCE, "free: freeing NULL block\n");
		DEBUG_TRAP();
	}
	if(p[-1] != CONST_MALLOCED){
		logPrintf(LOGM_RESOURCE, "free: block not allocated\n");
		DEBUG_TRAP();
	}
	free(p-1);
}

/* ------------------------------------------------------------------------- */

