/*
 * Name: notification.h
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-13
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: notification.h,v 1.1.1.1 1999/04/12 13:56:37 cs Exp $
 */

#ifndef __notification_H_INCLUDED__
#define __notification_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements a notification center class. A notification center is
a storage class that can store function invocations and call the functions
later when a specific event occurs. It provides for a kind of dynamic binding
at runtime between modules that don't know about each other when compiled.
*/

/* ------------------------------------------------------------------------- */

struct notifyAction;

typedef struct notificationCenter{
	btint				refCount;
    struct notifyAction	*table;
    btu32				tableSize;
    btu32				objectCount;
}notificationCenter_t;

/* ------------------------------------------------------------------------- */

notificationCenter_t	*ncNew(btu32 initialCapacity);
void	ncRetain(notificationCenter_t *nc);
void	ncRelease(notificationCenter_t *nc);
void	ncAddObserver(notificationCenter_t *nc, btcallback action, void *arg);
void	ncRemoveObserver(notificationCenter_t *nc, btcallback action, void *arg);
void	ncPostNotification(notificationCenter_t *nc, void *data);

/* ------------------------------------------------------------------------- */

#endif /* __notification_H_INCLUDED__ */