/*
 * Name: uidsecure.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-01-27
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: uidsecure.c,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */


#include "stdheaders.h"
#include "logging.h"
#include "uidsecure.h"
#include "util.h"
#include "system.h"

/* ------------------------------------------------------------------------- */

static uid_t	userUid = -1;
static uid_t	currentUid = -1;
static btbool	haveRootPermissions = 0;

/* ------------------------------------------------------------------------- */

void	uidsecInit(void)
{
	alarm(0);					/* can be inherited from parent process */
	userUid = getuid();
	currentUid = geteuid();
	haveRootPermissions = currentUid == 0 || userUid == 0;
	uidsecBecomeUser();
}

/* ------------------------------------------------------------------------- */

void	uidsecSetUserId(bti32 uid)
{
	if(currentUid == userUid){
		uidsecBecomeRoot();
		userUid = uid;
		uidsecBecomeUser();
	}else{
		userUid = uid;
	}
}

/* ------------------------------------------------------------------------- */

bti32	uidsecUserId(void)
{
	return userUid;
}

/* ------------------------------------------------------------------------- */

void uidsecBecomeRoot(void)
{
	if(!haveRootPermissions)	/* don't do anything if we are not root */
		return;
	if(currentUid == 0)			/* nothing to be done */
		return;
	if(sysSetEuid(0)){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "uidsecure: error becoming root: [%d] %s\n", errno, sysStrerror(errno));
	}
	currentUid = geteuid();	/* be safe and ask, sysSetEuid() may fail even without error return */
	if(currentUid != 0 && LOG_MODE_IS_ACTIVE(LOGM_ERROR)){
		logPrintf(LOGM_ERROR, "uidsecure: could not become root!\n");
	}
}

/* ------------------------------------------------------------------------- */

void uidsecBecomeUser(void)
{
	if(!haveRootPermissions)	/* don't do anything if we are not root */
		return;
	if(currentUid != 0 || userUid == 0)	/* nothing to be done */
		return;
	if(sysSetEuid(userUid)){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "uidsecure: error becoming user: [%d] %s\n", errno, sysStrerror(errno));
	}
	currentUid = geteuid();	/* be safe and ask, sysSetEuid() may fail even without error return */
	if(currentUid != userUid && LOG_MODE_IS_ACTIVE(LOGM_DEBUG)){
		logPrintf(LOGM_DEBUG, "uidsecure: could not become user!\n");
	}
}

/* ------------------------------------------------------------------------- */
