/*
 * Name: uidsecure.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-01-27
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: uidsecure.h,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */

#ifndef __UIDSECURE_H_INCLUDED__
#define __UIDSECURE_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module adds an extra layer of security for programs that must run under
root permissions. It allows the program to switch between privileged mode
(when it is needed) and non-privileged mode (used the rest of the time).
*/

/* ------------------------------------------------------------------------- */
void	uidsecInit(void);
/* This function initializes the module and sets the effective user-ID to the
 * real uid of the process. This makes set-uid-root programs run under user
 * permissions.
 */
/* ------------------------------------------------------------------------- */
void	uidsecSetUserId(bti32 uid);
/* This function changes the uid of the ordinary user for non-privileged
 * execution.
 */
/* ------------------------------------------------------------------------- */
bti32	uidsecUserId(void);
/* This function returns the uid of the non-privileged user. */
/* ------------------------------------------------------------------------- */
void	uidsecBecomeRoot(void);
/* This function sets the effective user-ID to 0 (root permissions) */
/* ------------------------------------------------------------------------- */
void	uidsecBecomeUser(void);
/* This function sets the effective user-ID to the user's uid (either the
 * real uid of the user that executes the program or the user that was set
 * with 'uidsecSetUserId()'.
 */
/* ------------------------------------------------------------------------- */

#endif /* __UIDSECURE_H_INCLUDED__ */
