/*
 * Name: unicode.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-22
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: unicode.h,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */

#ifndef __unicode_H_INCLUDED__
#define __unicode_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
General toolset for working with unicode strings: conversion from/to ASCII,
basic string functions. It also contains some operations on ASCII strings.
*/

#define	UNI_MAX_ASCIIMAP	4	/* should be 4*n */

typedef struct uniProperty{
	btuni	upcase;
	btuni	lowcase;
	btu8	serverRep[UNI_MAX_ASCIIMAP];
	btu8	clientRep[UNI_MAX_ASCIIMAP];
}uniProperty_t;

/* ------------------------------------------------------------------------- */
/* --------------------------- unicode constants --------------------------- */
/* ------------------------------------------------------------------------- */

#define	UNI_CHAR_BACKSLASH		((btuni)'\\')
#define	UNI_CHAR_SLASH			((btuni)'/')
#define	UNI_CHAR_DOT			((btuni)'.')


/* ------------------------------------------------------------------------- */
/* --------------------------- unicode parsing ----------------------------- */
/* ------------------------------------------------------------------------- */

btbool	uniParseMappingFile(btu8 *fileName, btbool isServer, uniProperty_t *uniTable, btuni *uniErrorChar, btu8 *asciiErrorChar, btsize *asciiErrorLen);
btbool	uniParseCaseConvertFile(btu8 *fileName, uniProperty_t *uniTable);

/* ------------------------------------------------------------------------- */
/* -------------------------- unicode operations --------------------------- */
/* ------------------------------------------------------------------------- */

btuni	*uniStrDup(btuni *s);
void	uniStrAddHash(btu32 *crcp, btuni *s, btbool isCaseSensitive);
btuni	uniCharToupper(btuni c);
btuni	uniCharTolower(btuni c);
btu8	asciiCharToupper(btu8 c);
btu8	asciiCharTolower(btu8 c);
void	uniStrCpy(btuni *dest, btuni *src, btsize arrSize);
void	asciiStrCpy(btu8 *dest, btu8 *src, btsize arrSize);

static inline btsize	uniStrLen(btuni *s)
{
btsize	l = 0;

	while(s[l])	l++;
	return l;
}

static inline btu32	uniStrHash(btuni *s, btbool isCaseSensitive)
{
btu32	crc = -1;

	uniStrAddHash(&crc, s, isCaseSensitive);
	return crc;
}

static inline int	uniStrCmp(btuni *s1, btuni *s2, btbool caseSensitive)
{
btuni	c1, c2;

	if(caseSensitive){
		while(*s1 != 0 && *s2 != 0){
			if(*s1 < *s2)	return -1;
			if(*s1 > *s2)	return 1;
			s1++; s2++;
		}
	}else{
		while(*s1 != 0 && *s2 != 0){
			c1 = uniCharToupper(*s1);
			c2 = uniCharToupper(*s2);
			if(c1 < c2)
				return -1;
			if(c1 > c2)
				return 1;
			s1++; s2++;
		}
	}
	if(*s2 != 0)
		return -1;
	if(*s1 != 0)
		return 1;
	return 0;
}

static inline void	uniStrToupper(btuni *s)
{
	while(*s != 0){
		*s = uniCharToupper(*s);
		s++;
	}
}

static inline void	uniStrTolower(btuni *s)
{
	while(*s != 0){
		*s = uniCharTolower(*s);
		s++;
	}
}

/* ------------------------------------------------------------------------- */
/* -------------------------- unicode conversions -------------------------- */
/* ------------------------------------------------------------------------- */

/* initialisation: */

void	uniInit(btu8 *caseConvertPath, btu8 *serverMapPath, btu8 *clientMapPath, btsize maxBlocks);
void	uniTrivialInit(void);

/* basic conversion operations: */

btuni	convCharLocalAsciiToUni(btu8 **string);
btuni	convCharServerAsciiToUni(btu8 **string);
btbool	convCharUniToLocalAscii(btuni c, btu8 **string);
btbool	convCharUniToServerAscii(btuni c, btu8 **string);

/* derived conversion operations: */

void	convLocalAsciiToUni(btu8 *ascii, btuni *uni, btsize arrSize);
void	convUniToLocalAscii(btuni *uni, btu8 *ascii, btsize arrSize);
void	convServerAsciiToUni(btu8 *ascii, btuni *uni, btsize arrSize);
void	convUniToServerAscii(btuni *uni, btu8 *ascii, btsize arrSize);

/* intended for debug outputs: */
btu8	*convUniToAscii(btuni *uni);

/* ------------------------------------------------------------------------- */

#endif	/* __unicode_H_INCLUDED__ */
