/*
 * Name: sunrpc.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-12-03
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: sunrpc.h,v 1.1.1.1 1999/04/12 13:56:45 cs Exp $
 */

#ifndef __sunrpc_H_INCLUDED__
#define __sunrpc_H_INCLUDED__

#include "basictypes.h"
#include "fifo.h"
#include "nfs_prot.h"

/*
General Description:
This module contains marshalling and unmarshalling of the NFS RPC protocol,
the underlying SUN RPC protocol and (necessarily) basic data types according
to the RFC1014 XDR specification.
*/

/* ------------------------------------------------------------------------- */

EXC_DECLARE(srpcExcEncodeOverflow);
EXC_DECLARE(srpcExcDecodeOverflow);
EXC_DECLARE(srpcExcUnexpected);

#define	SRPC_MAX_BUFFER		8800

typedef struct srpcMsg{
	fifoObject_t	fifo;		/* for fifo of allocated objects */
	btsize			size;		/* used size in bytes */
	btsize			readIndex;
	btint			usageCount;
	btu8			data[SRPC_MAX_BUFFER];
}srpcMsg_t;

typedef struct srpcRequestHeader{
	btu32	xid;
	btu32	rpcvers;	/* == 2 for current version of RPC (--> MSG_DENIED) */
	btu32	prog;		/* == 100003 (--> PROG_UNAVAIL) */
	btu32	vers;		/* == 2 (--> PROG_MISMATCH) */
	btu32	proc;		/* --> PROC_UNAVAIL */
	btu32	credFlavor;
	btu32	credUid;
	btu32	credGid;
	btu32	verfFlavor;
	void	*verfData;
	btu32	verfLen;
}srpcRequestHeader_t;

/* ------------------------------------------------------------------------- */

srpcMsg_t	*srpcMsgNew(void);
void		srpcMsgFree(srpcMsg_t *msg);

/* ------------------------------------------------------------------------- */

void	srpcEncodeRpcHeader(struct srpcMsg *msg, btu32 xid, btu32 isDenied, btu32 status);
void	srpcDecodeRpcHeader(struct srpcMsg *msg, srpcRequestHeader_t *hdr);

void	srpcEncodeAttrstat(struct srpcMsg *msg, attrstat *as);
void	srpcDecodeSattrargs(struct srpcMsg *msg, sattrargs *sa);
void	srpcDecodeDiropargs(struct srpcMsg *msg, diropargs *da);
void	srpcEncodeDiropres(struct srpcMsg *msg, diropres *dr);
void	srpcEncodeReadlinkres(struct srpcMsg *msg, readlinkres *rr);
void	srpcDecodeReadargs(struct srpcMsg *msg, readargs *ra);
void	srpcEncodeReadres(struct srpcMsg *msg, readres *rr);
void	srpcDecodeWriteargs(struct srpcMsg *msg, writeargs *wa);
void	srpcDecodeCreateargs(struct srpcMsg *msg, createargs *ca);
void	srpcDecodeRenameargs(struct srpcMsg *msg, renameargs *ra);
void	srpcDecodeLinkargs(struct srpcMsg *msg, linkargs *la);
void	srpcDecodeSymlinkargs(struct srpcMsg *msg, symlinkargs *sa);
void	srpcDecodeReaddirargs(struct srpcMsg *msg, readdirargs *ra);
void	srpcEncodeReaddirres(struct srpcMsg *msg, readdirres *rr);
void	srpcEncodeStatfsres(struct srpcMsg *msg, statfsres *sr);
void	srpcEncodeNfsstat(struct srpcMsg *msg, nfsstat *s);
void	srpcDecodeNfs_fh(struct srpcMsg *msg, nfs_fh *fh);

/* ------------------------------------------------------------------------- */

#endif /* __sunrpc_H_INCLUDED__ */
