/*
 * Name: module.c
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-10
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: module.c,v 1.1.1.1 1999/04/12 13:56:43 cs Exp $
 */

#include "stdheaders.h"
#include "module.h"
#include "array.h"
#include "hashtable.h"
#include "notification.h"
#include "util.h"

/* ------------------------------------------------------------------------- */

static hashtable_t			*allModules;
static notificationCenter_t	*registerNotification;

/* ------------------------------------------------------------------------- */

static btint	moduleCompare(void *o1, void *o2)
{
module_t	*m1 = o1, *m2 = o2;

	return strcmp(m1->name, m2->name);
}

static btu32	moduleHashVal(void *obj)
{
module_t	*this = obj;

	return utilStrHash(this->name, 1);
}

static collectionObjectI_t	moduleInterface = {
	moduleCompare,
	moduleHashVal,
	NULL,
	NULL
};

/* ------------------------------------------------------------------------- */

void	modRegister(module_t *mod)
{
	if(allModules == NULL){
		allModules = htabNew(&moduleInterface, 8, 75);
	}
	htabAddObject(allModules, mod);
	if(registerNotification != NULL){
		ncPostNotification(registerNotification, mod);
	}
}

/* ------------------------------------------------------------------------- */

struct notificationCenter	*modRegisterNotification(void)
{
	if(registerNotification == NULL){
		registerNotification = ncNew(4);
	}
	return registerNotification;
}

/* ------------------------------------------------------------------------- */

struct array	*modAllModules(void)
{
	return htabAllObjects(allModules);
}

/* ------------------------------------------------------------------------- */

module_t	*modFind(btu8 *name)
{
	return htabFindObjectEqualTo(allModules, &name);
}

/* ------------------------------------------------------------------------- */
