/*
 * Name: guiLogging.c
 * Project: CIFS Client GUI
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-12-20
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: guiLogging.c,v 1.1.1.1 1999/04/12 13:56:40 cs Exp $
 */

#include "stdheaders.h"
#include "logging.h"
#include "guiHelper.h"

/* ------------------------------------------------------------------------- */

bti32	logModeMask = (1 << LOGM_ERROR) | (1 << LOGM_INFO);
int		logDestination = LOG_DEST_CMDLINE;

/* ------------------------------------------------------------------------- */

void	logError(char *errTitle, char *errMsg)
{
char			*p, *errContext = NULL;
int				len;

	len = strlen(errMsg);
	if(len > 0 && errMsg[len-1] == '\n')	/* remove trailing NL */
		errMsg[len-1] = 0;
	if(memcmp(errMsg, "LOC: ", 5) == 0){
		errContext = "local machine";
	}else if(memcmp(errMsg, "DOS: ", 5) == 0){
		errContext = "DOS error on server";
	}else if(memcmp(errMsg, "SRV: ", 5) == 0){
		errContext = "server";
	}else if(memcmp(errMsg, "HRD: ", 5) == 0){
		errContext = "hard error on server";
	}else if(memcmp(errMsg, "SSL: ", 5) == 0){
		errContext = "SSL (Secure Socket Layer)";
	}else if(memcmp(errMsg, "FRW: ", 5) == 0){
		errContext = "local machine (framework)";
	}else if(memcmp(errMsg, "UNIX: ", 6) == 0){
		errContext = "local machine (Unix error message)";
	}
	if(errContext != NULL){
		p = strchr(errMsg, ':') + 2;
		guiRunAlert(NULL, NULL, "OK", errTitle, "%s\nError Context: %s", p, errContext);
	}else{
		guiRunAlert(NULL, NULL, "OK", errTitle, "%s", errMsg);
	}
}

static void	logVprintf(int mode, const char *format, va_list vlist)
{
static char				buffer[2048];
static char				lastErr[2048] = {0};
static struct timeval	lastErrTime;
struct timeval			now;
struct timezone			tz;
char					*ctx;
btint					age;

	vsprintf(buffer, format, vlist);
	if(strlen(buffer) >= sizeof(buffer)){
		guiRunAlert(NULL, NULL, " Terminate ", "Fatal Error", "Buffer overflow in logging!");
		exit(1);
	}
	if(strncmp(buffer, lastErr, sizeof(lastErr)) == 0){	/* same error text */
		gettimeofday(&now, &tz);
		age = now.tv_sec - lastErrTime.tv_sec;
		age *= 10;
		age += (now.tv_usec - lastErrTime.tv_usec)/100000;
		if(age < 4 && age >= 0)	/* don't log if it's the same message within 0.4 seconds */
			return;
	}
	strncpy(lastErr, buffer, sizeof(lastErr));
	switch(mode){
		case LOGM_INFO:		ctx = "Info";		break;
		case LOGM_WARN:		ctx = "Warning";	break;
		case LOGM_ERROR:	ctx = "Error";		break;
		default:			ctx = "Debug";		break;
	}
	logError(ctx, buffer);
	gettimeofday(&lastErrTime, &tz);
}

/* ------------------------------------------------------------------------- */

void	logPrintf(int mode, const char *format, ...)
{
va_list		vlist;

	va_start(vlist, format);
	logVprintf(mode, format, vlist);
	va_end(vlist);
}

/* ------------------------------------------------------------------------- */

