/*
 * Name: htmlDisplay.c
 * Project: CIFS Client GUI
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-12-19
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: htmlDisplay.c,v 1.2 1999/05/30 21:19:57 cs Exp $
 */

#include "basictypes.h"
#include <ctype.h>
#include <string.h>
#include <gtk/gtk.h>
#include "guiHelper.h"
#include "htmlDisplay.h"

static gchar *tags[] = {
	"p", "br", "b", "em", "sl", "small", "normalsize", "big", "large", "huge"
};

#define	TAG_PARAGRAPH	0
#define	TAG_BREAK		1
#define	TAG_BOLD		2
#define	TAG_EM			3
#define	TAG_SL			4
#define	TAG_SMALL		5
#define	TAG_NORMAL		6
#define	TAG_BIG			7
#define	TAG_LARGE		8
#define	TAG_HUGE		9

static gint	parseTag(gchar **pp, btbool *isEndTag)
{
gchar	*p = *pp, buffer[32];
gint	i;

	*isEndTag = 0;
	if(*p == '<')
		p++;
	if(*p == '/'){
		p++;
		*isEndTag = 1;
	}
	for(i=0;i<sizeof(buffer)-1;i++){
		if(p[i] == 0 || isspace(p[i]) || p[i] == '>')
			break;
		buffer[i] = p[i];
	}
	buffer[i] = 0;
	while(p[i] != '>' && p[i] != 0)
		i++;
	if(p[i] == '>')
		i++;
	*pp = &p[i];
	for(i=0;i<sizeof(tags)/sizeof(tags[0]);i++){
		if(strcasecmp(buffer, tags[i]) == 0)
			return i;
	}
	return -1;
}

void	htmlInsertText(GtkText *text, bti8 *html)
{
gchar	*p, *rangeStart;
btbool	isEndTag, currentBold, currentEm, skipspace = 0;
gint	tag, currentSize;
GdkFont	*currentFont;

	currentSize = GUI_FONT_NORMAL;
	currentEm = currentBold = FALSE;
	currentFont = guiGetFont(currentSize, currentBold, currentEm);
	for(rangeStart=p=html;*p!=0;){
		if(*p == '<'){
			if(p != rangeStart)
				gtk_text_insert(text, currentFont, NULL, NULL, rangeStart, p - rangeStart);
			tag = parseTag(&p, &isEndTag);
			rangeStart = p;
			switch(tag){
			case TAG_PARAGRAPH:
				gtk_text_insert(text, currentFont, NULL, NULL, "\n\n", 2);
				skipspace = 1;
				rangeStart = p;
				break;
			case TAG_BREAK:
				gtk_text_insert(text, currentFont, NULL, NULL, "\n", 1);
				skipspace = 1;
				rangeStart = p;
				break;
			case TAG_BOLD:
				currentBold = !isEndTag;
				currentFont = guiGetFont(currentSize, currentBold, currentEm);
				break;
			case TAG_EM:
			case TAG_SL:
				currentEm = !isEndTag;
				currentFont = guiGetFont(currentSize, currentBold, currentEm);
				break;
			case TAG_SMALL:
			case TAG_NORMAL:
			case TAG_BIG:
			case TAG_LARGE:
			case TAG_HUGE:
				currentSize = isEndTag ? GUI_FONT_NORMAL : (GUI_FONT_SMALL + tag - TAG_SMALL);
				currentFont = guiGetFont(currentSize, currentBold, currentEm);
				break;
			}
		}else if(isspace(*p)){
			if(!skipspace){
				gtk_text_insert(text, currentFont, NULL, NULL, rangeStart, p - rangeStart);
				gtk_text_insert(text, currentFont, NULL, NULL, " ", 1);
				skipspace = 1;
			}
			p++;
			rangeStart = p; 
		}else{
			if(skipspace){
				skipspace = 0;
				rangeStart = p;
			}
			p++;	/* skip the character, it'll be added in it's range */
		}
	}
	if(p > rangeStart){
		gtk_text_insert(text, currentFont, NULL, NULL, rangeStart, p - rangeStart);
	}
}
