/*
 * Name: maininterface.h
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-10
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: maininterface.h,v 1.5 1999/06/10 20:35:27 cs Exp $
 */

#ifndef __maininterface_H_INCLUDED__
#define __maininterface_H_INCLUDED__

/*
General Description:
This header contains constant definitions that are shared between the main
framework (the main module) and the GUI or commandline applications. It is
therefore included by both, the daemon and the UI clients.
*/

/* ------------------------------------------------------------------------- */

#define	MAIN_IPC_SOCKET_PATH	"/tmp/sharity.sock"

#define	MAIN_IPC_PROTOCOL_VERSION	3

/* for IPC calls GUI -> daemon: */
#define	MAIN_IPCFN_LIST_MODULES		0
#define	MAIN_IPCFN_SLASH_CHARS		1
#define	MAIN_IPCFN_MOUNT			2
#define	MAIN_IPCFN_REGISTERCB		3
#define	MAIN_IPCFN_UNREGISTERCB		4
#define	MAIN_IPCFN_GET_OBDES		5
#define	MAIN_IPCFN_CONFIG_OBJECTS	6
#define	MAIN_IPCFN_DSOURCE_EXEC		7
#define	MAIN_IPCFN_GET_CFGVAL		8
#define	MAIN_IPCFN_SET_CFGVAL		9
#define	MAIN_IPCFN_DEL_CFGVAL		10
#define	MAIN_IPCFN_UMOUNT			11
#define	MAIN_IPCFN_STOREMNT			12
#define	MAIN_IPCFN_LOOKUP_LIST		13
#define	MAIN_IPCFN_HELP_FOR_KEY		14
#define	MAIN_IPCFN_WILL_TERMINATE	15

#define	MAIN_IPCFN_NUMFUNCTIONS		16	/* number of functions defined */

/* for IPC calls daemon -> GUI: */
#define	MAIN_IPCCFN_RUN_DIALOG		0	/* arg: objectId, flag */
#define	MAIN_IPCCFN_CLOSE_DIALOG	1	/* arg: objectId */
#define	MAIN_IPCCFN_NOTIFY_UPDATE	2	/* arg: list of IDs (prefix=count) */
#define	MAIN_IPCCFN_RUN_FILESELECTOR 3	/* arg: title, path -> isSet, path */
#define	MAIN_IPCCFN_RUN_ALERT		4	/* arg: title, text, button1..3 */
#define	MAIN_IPCCFN_RUN_SELECTION	5	/* arg: title, ok, cancel, [elts] */
#define	MAIN_IPCCFN_DISPLAY_ERROR	6	/* arg: errclass, errcode, errtxt */
#define	MAIN_IPCCFN_ASK_PASSWORD	7
#define	MAIN_IPCCFN_ASK_DATA		8

/* values for GUI config object types: */
#define	MAIN_GUI_CLASS_PLAINCFG		0
#define	MAIN_GUI_CLASS_HIERARCHYCFG	1
#define	MAIN_GUI_CLASS_LIST			2

/* values for GUI guiEntry accessPermissions: */
#define	MAIN_GUI_PERM_ANYBODY		(1<<0)
#define	MAIN_GUI_PERM_EXPERT		(1<<1)	/* can be combined with others */
#define	MAIN_GUI_PERM_AUTHORIZED	(1<<2)
#define	MAIN_GUI_PERM_ROOT			(1<<3)

/* values for entry types: */
#define	MAIN_GUI_ENTRY_STRING		0	/* descriptor = NULL */
#define	MAIN_GUI_ENTRY_PASSWORD		1	/* descriptor = NULL */
#define	MAIN_GUI_ENTRY_FILE			2	/* descriptor = NULL */
#define	MAIN_GUI_ENTRY_ENUM			3	/* descriptor = guiTypedesEnum_t */
#define	MAIN_GUI_ENTRY_BOOL			4	/* descriptor = NULL */
#define	MAIN_GUI_ENTRY_NUMBER		5	/* descriptor = guiTypedesNumeric_t */
#define	MAIN_GUI_ENTRY_SEPARATOR	6	/* descriptor = NULL */

/* values for entry flags: */
#define	MAIN_GUI_ENTRY_FLG_DEFAULT_RESPONDER	(1<<0)	/* put cursor into this entry initially */

/* values for data source method indices: */
#define	MAIN_GUI_METHOD_GET			0	/* [hierarchyItem] isDefault */
	/* list result: per row: row ID (string), per column: data (string) */
#define	MAIN_GUI_METHOD_SET			1	/* [hierarchyItem] .... */
#define	MAIN_GUI_METHOD_HLIST		2	/* listAll */
#define	MAIN_GUI_METHOD_ADDHITEM	3	/* itemName */
#define	MAIN_GUI_METHOD_DELHITEM	4	/* itemName */

#define	MAIN_GUI_METHOD_LOGIN		1
#define	MAIN_GUI_METHOD_LOGOUT		2
#define	MAIN_GUI_METHOD_STORELOGIN	3
#define	MAIN_GUI_METHOD_DELSTORED	4

#define	MAIN_GUI_METHOD_MOUNT		1
#define	MAIN_GUI_METHOD_UMOUNT		2
#define	MAIN_GUI_METHOD_STOREMOUNT	3

#define	MAIN_GUI_METHOD_CANCEL		2	/* cancel method for dialogs */

/* values for button flags (bitmask): */
#define	MAIN_GUI_BUTTON_ACTIVE_IF_DIRTY	1	/* only active if entries dirty */
#define	MAIN_GUI_BUTTON_CLOSE_DIALOG	2	/* this button closes the dialog */
#define	MAIN_GUI_BUTTON_EQUALS_CLOSEBOX	4	/* this is the cancel button */
#define	MAIN_GUI_BUTTON_DEFAULT			8	/* this button is default */


/* published lists: */
#define	MAIN_LIST_MOUNTS			"mainMountList"

/* Reasons for MAIN_IPCFN_WILL_TERMINATE: */
#define	MAIN_TERM_USRQUIT		0	/* user quits application */
#define	MAIN_TERM_LOGOUT		1	/* app killed because user logged out */
#define	MAIN_TERM_CRASH			2	/* app crahsed */

/* Flag values for dialogs: */
#define	MAIN_DIALOG_FLG_MODAL	(1<<0)	/* run this dialog modally (advisory)*/

/* ------------------------------------------------------------------------- */

#endif /* __maininterface_H_INCLUDED__ */
